@@ -1,34 +1,8 @@
 #!/usr/bin/env python
-# $Id: unix_daemon.py 1174 2013-02-19 11:25:49Z g.rodola $
 
-#  pyftpdlib is released under the MIT license, reproduced below:
-#  ======================================================================
-#  Copyright (C) 2007-2013 Giampaolo Rodola' <g.rodola@gmail.com>
-#
-#                         All Rights Reserved
-#
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation
-# files (the "Software"), to deal in the Software without
-# restriction, including without limitation the rights to use,
-# copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following
-# conditions:
-#
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
-#
-#  ======================================================================
+# Copyright (C) 2007-2016 Giampaolo Rodola' <g.rodola@gmail.com>.
+# Use of this source code is governed by MIT license that can be
+# found in the LICENSE file.
 
 """A basic unix daemon using the python-daemon library:
 http://pypi.python.org/pypi/python-daemon
@@ -56,20 +30,18 @@ Authors:
  - Giampaolo Rodola' - g.rodola <at> gmail.com
 """
 
-from __future__ import with_statement
-
-import os
+import atexit
 import errno
+import optparse
+import os
+import signal
 import sys
 import time
-import optparse
-import signal
-import atexit
 
-from pyftpdlib.handlers import FTPHandler
-from pyftpdlib.servers import FTPServer
 from pyftpdlib.authorizers import UnixAuthorizer
 from pyftpdlib.filesystems import UnixFilesystem
+from pyftpdlib.handlers import FTPHandler
+from pyftpdlib.servers import FTPServer
 
 
 # overridable options
@@ -79,31 +51,28 @@ PID_FILE = "/var/run/pyftpdlib.pid"
 LOG_FILE = "/var/log/pyftpdlib.log"
 WORKDIR = os.getcwd()
 UMASK = 0
-    
-def print_(s):
-    sys.stdout.write(s + '\n')
-    sys.stdout.flush()
+
 
 def pid_exists(pid):
     """Return True if a process with the given PID is currently running."""
     try:
         os.kill(pid, 0)
-    except OSError:
-        err = sys.exc_info()[1]
+    except OSError as err:
         return err.errno == errno.EPERM
     else:
         return True
 
+
 def get_pid():
     """Return the PID saved in the pid file if possible, else None."""
     try:
         with open(PID_FILE) as f:
             return int(f.read().strip())
-    except IOError:
-        err = sys.exc_info()[1]
+    except IOError as err:
         if err.errno != errno.ENOENT:
             raise
 
+
 def stop():
     """Keep attempting to stop the daemon for 5 seconds, first using
     SIGTERM, then using SIGKILL.
@@ -118,10 +87,9 @@ def stop():
         sys.stdout.flush()
         try:
             os.kill(pid, sig)
-        except OSError:
-            e = sys.exc_info()[1]
-            if e.errno == errno.ESRCH:
-                print_("\nstopped (pid %s)" % pid)
+        except OSError as err:
+            if err.errno == errno.ESRCH:
+                print("\nstopped (pid %s)" % pid)
                 return
             else:
                 raise
@@ -132,15 +100,17 @@ def stop():
             sys.exit("\ncould not kill daemon (pid %s)" % pid)
         time.sleep(0.1)
 
+
 def status():
     """Print daemon status and exit."""
     pid = get_pid()
     if not pid or not pid_exists(pid):
-        print_("daemon not running")
+        print("daemon not running")
     else:
-        print_("daemon running with pid %s" % pid)
+        print("daemon running with pid %s" % pid)
     sys.exit(0)
 
+
 def get_server():
     """Return a pre-configured FTP server instance."""
     handler = FTPHandler
@@ -149,25 +119,26 @@ def get_server():
     server = FTPServer((HOST, PORT), handler)
     return server
 
+
 def daemonize():
     """A wrapper around python-daemonize context manager."""
     def _daemonize():
-        pid = os.fork() 
+        pid = os.fork()
         if pid > 0:
             # exit first parent
-            sys.exit(0) 
-    
+            sys.exit(0)
+
         # decouple from parent environment
-        os.chdir(WORKDIR) 
-        os.setsid() 
-        os.umask(0) 
-    
-        # do second fork     
-        pid = os.fork() 
+        os.chdir(WORKDIR)
+        os.setsid()
+        os.umask(0)
+
+        # do second fork
+        pid = os.fork()
         if pid > 0:
             # exit from second parent
-            sys.exit(0) 
-    
+            sys.exit(0)
+
         # redirect standard file descriptors
         sys.stdout.flush()
         sys.stderr.flush()
@@ -177,12 +148,11 @@ def daemonize():
         os.dup2(si.fileno(), sys.stdin.fileno())
         os.dup2(so.fileno(), sys.stdout.fileno())
         os.dup2(se.fileno(), sys.stderr.fileno())
-    
+
         # write pidfile
         pid = str(os.getpid())
-        f = open(PID_FILE,'w')
-        f.write("%s\n" % pid)
-        f.close()
+        with open(PID_FILE, 'w') as f:
+            f.write("%s\n" % pid)
         atexit.register(lambda: os.remove(PID_FILE))
 
     pid = get_pid()
@@ -190,7 +160,7 @@ def daemonize():
         sys.exit('daemon already running (pid %s)' % pid)
     # instance FTPd before daemonizing, so that in case of problems we
     # get an exception here and exit immediately
-    server = get_server()    
+    server = get_server()
     _daemonize()
     server.serve_forever()
 
