@@ -1,33 +1,6 @@
-#!/usr/bin/env python
-# $Id: ioloop.py 1217 2013-04-18 18:21:44Z g.rodola $
-
-#  ======================================================================
-#  Copyright (C) 2007-2013 Giampaolo Rodola' <g.rodola@gmail.com>
-#
-#                         All Rights Reserved
-#
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation
-# files (the "Software"), to deal in the Software without
-# restriction, including without limitation the rights to use,
-# copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following
-# conditions:
-#
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
-#
-#  ======================================================================
+# Copyright (C) 2007-2016 Giampaolo Rodola' <g.rodola@gmail.com>.
+# Use of this source code is governed by MIT license that can be
+# found in the LICENSE file.
 
 """
 A specialized IO loop on top of asyncore adding support for epoll()
@@ -83,30 +56,51 @@ server = Server('localhost', 8021)
 IOLoop.instance().loop()
 """
 
-import asyncore
 import asynchat
+import asyncore
 import errno
-import select
-import os
-import sys
-import traceback
-import time
 import heapq
+import os
+import select
 import socket
-import logging
+import sys
+import time
+import traceback
 try:
     import threading
 except ImportError:
     import dummy_threading as threading
 
-from pyftpdlib._compat import MAXSIZE, callable, b
-from pyftpdlib.log import logger, _config_logging
+from ._compat import callable
+from .log import config_logging
+from .log import debug
+from .log import is_logging_configured
+from .log import logger
 
 
 timer = getattr(time, 'monotonic', time.time)
 _read = asyncore.read
 _write = asyncore.write
 
+# These errnos indicate that a connection has been abruptly terminated.
+_ERRNOS_DISCONNECTED = set((
+    errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN, errno.ECONNABORTED,
+    errno.EPIPE, errno.EBADF))
+if hasattr(errno, "WSAECONNRESET"):
+    _ERRNOS_DISCONNECTED.add(errno.WSAECONNRESET)
+if hasattr(errno, "WSAECONNABORTED"):
+    _ERRNOS_DISCONNECTED.add(errno.WSAECONNABORTED)
+
+# These errnos indicate that a non-blocking operation must be retried
+# at a later time.
+_ERRNOS_RETRY = set((errno.EAGAIN, errno.EWOULDBLOCK))
+if hasattr(errno, "WSAEWOULDBLOCK"):
+    _ERRNOS_RETRY.add(errno.WSAEWOULDBLOCK)
+
+
+class RetryError(Exception):
+    pass
+
 
 # ===================================================================
 # --- scheduler
@@ -148,8 +142,9 @@ class _Scheduler(object):
         # remove cancelled tasks and re-heapify the queue if the
         # number of cancelled tasks is more than the half of the
         # entire queue
-        if self._cancellations > 512 \
-          and self._cancellations > (len(self._tasks) >> 1):
+        if (self._cancellations > 512 and
+                self._cancellations > (len(self._tasks) >> 1)):
+            debug("re-heapifying %s cancelled tasks" % self._cancellations)
             self.reheapify()
 
         try:
@@ -182,8 +177,8 @@ class _CallLater(object):
 
     def __init__(self, seconds, target, *args, **kwargs):
         assert callable(target), "%s is not callable" % target
-        assert MAXSIZE >= seconds >= 0, "%s is not greater than or equal " \
-                                        "to 0 seconds" % seconds
+        assert sys.maxsize >= seconds >= 0, \
+            "%s is not greater than or equal to 0 seconds" % seconds
         self._delay = seconds
         self._target = target
         self._args = args
@@ -210,9 +205,9 @@ class _CallLater(object):
             sig = object.__repr__(self)
         else:
             sig = repr(self._target)
-        sig += ' args=%s, kwargs=%s, cancelled=%s, secs=%s' \
-                % (self._args or '[]',  self._kwargs or '{}', self.cancelled,
-                   self._delay)
+        sig += ' args=%s, kwargs=%s, cancelled=%s, secs=%s' % (
+            self._args or '[]', self._kwargs or '{}', self.cancelled,
+            self._delay)
         return '<%s>' % sig
 
     __str__ = __repr__
@@ -226,14 +221,13 @@ class _CallLater(object):
         assert not self.cancelled, "already cancelled"
         exc = None
         try:
-            try:
-                self._target(*self._args, **self._kwargs)
-            except Exception:
-                exc = sys.exc_info()[1]
-                if self._errback is not None:
-                    self._errback()
-                else:
-                    raise
+            self._target(*self._args, **self._kwargs)
+        except Exception as _:
+            exc = _
+            if self._errback is not None:
+                self._errback()
+            else:
+                raise
         finally:
             self._post_call(exc)
 
@@ -245,10 +239,10 @@ class _CallLater(object):
 
     def cancel(self):
         """Unschedule this call."""
-        assert not self.cancelled, "already cancelled"
-        self.cancelled = True
-        self._target = self._args = self._kwargs = self._errback = None
-        self._sched.unregister(self)
+        if not self.cancelled:
+            self.cancelled = True
+            self._target = self._args = self._kwargs = self._errback = None
+            self._sched.unregister(self)
 
 
 class _CallEvery(_CallLater):
@@ -276,16 +270,27 @@ class _IOLoop(object):
         self.socket_map = {}
         self.sched = _Scheduler()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        self.close()
+
+    def __repr__(self):
+        status = [self.__class__.__module__ + "." + self.__class__.__name__]
+        status.append("(fds=%s, tasks=%s)" % (
+            len(self.socket_map), len(self.sched._tasks)))
+        return '<%s at %#x>' % (' '.join(status), id(self))
+
+    __str__ = __repr__
+
     @classmethod
     def instance(cls):
         """Return a global IOLoop instance."""
         if cls._instance is None:
-            cls._lock.acquire()
-            try:
+            with cls._lock:
                 if cls._instance is None:
                     cls._instance = cls()
-            finally:
-                cls._lock.release()
         return cls._instance
 
     def register(self, fd, instance, events):
@@ -320,18 +325,17 @@ class _IOLoop(object):
         """
         if not _IOLoop._started_once:
             _IOLoop._started_once = True
-            if not logging.getLogger().handlers:
+            if not is_logging_configured():
                 # If we get to this point it means the user hasn't
                 # configured logging. We want to log by default so
                 # we configure logging ourselves so that it will
                 # print to stderr.
-                _config_logging()
+                config_logging()
 
         if blocking:
             # localize variable access to minimize overhead
             poll = self.poll
             socket_map = self.socket_map
-            tasks = self.sched._tasks
             sched_poll = self.sched.poll
 
             if timeout is not None:
@@ -373,6 +377,7 @@ class _IOLoop(object):
 
     def close(self):
         """Closes the IOLoop, freeing any resources used."""
+        debug("closing IOLoop", self)
         self.__class__._instance = None
 
         # free connections
@@ -380,9 +385,8 @@ class _IOLoop(object):
         for inst in instances:
             try:
                 inst.close()
-            except OSError:
-                err = sys.exc_info()[1]
-                if err.args[0] != errno.EBADF:
+            except OSError as err:
+                if err.errno != errno.EBADF:
                     logger.error(traceback.format_exc())
             except Exception:
                 logger.error(traceback.format_exc())
@@ -422,7 +426,7 @@ class Select(_IOLoop):
         try:
             del self.socket_map[fd]
         except KeyError:
-            pass
+            debug("call: unregister(); fd was no longer in socket_map", self)
         for l in (self._r, self._w):
             try:
                 l.remove(fd)
@@ -434,13 +438,14 @@ class Select(_IOLoop):
         if inst is not None:
             self.unregister(fd)
             self.register(fd, inst, events)
+        else:
+            debug("call: modify(); fd was no longer in socket_map", self)
 
     def poll(self, timeout):
         try:
             r, w, e = select.select(self._r, self._w, [], timeout)
-        except select.error:
-            err = sys.exc_info()[1]
-            if err.args[0] == errno.EINTR:
+        except select.error as err:
+            if err.errno == errno.EINTR:
                 return
             raise
 
@@ -462,8 +467,9 @@ class Select(_IOLoop):
 # ===================================================================
 
 class _BasePollEpoll(_IOLoop):
-    """This is common to both poll/epoll implementations which
-    almost share the same interface.
+    """This is common to both poll() (UNIX), epoll() (Linux) and
+    /dev/poll (Solaris) implementations which share almost the same
+    interface.
     Not supposed to be used directly.
     """
 
@@ -472,26 +478,48 @@ class _BasePollEpoll(_IOLoop):
         self._poller = self._poller()
 
     def register(self, fd, instance, events):
-        self._poller.register(fd, events)
+        try:
+            self._poller.register(fd, events)
+        except EnvironmentError as err:
+            if err.errno == errno.EEXIST:
+                debug("call: register(); poller raised EEXIST; ignored", self)
+            else:
+                raise
         self.socket_map[fd] = instance
 
     def unregister(self, fd):
         try:
             del self.socket_map[fd]
         except KeyError:
-            pass
+            debug("call: unregister(); fd was no longer in socket_map", self)
         else:
-            self._poller.unregister(fd)
+            try:
+                self._poller.unregister(fd)
+            except EnvironmentError as err:
+                if err.errno in (errno.ENOENT, errno.EBADF):
+                    debug("call: unregister(); poller returned %r; "
+                          "ignoring it" % err, self)
+                else:
+                    raise
 
     def modify(self, fd, events):
-        self._poller.modify(fd, events)
+        try:
+            self._poller.modify(fd, events)
+        except OSError as err:
+            if err.errno == errno.ENOENT and fd in self.socket_map:
+                # XXX - see:
+                # https://github.com/giampaolo/pyftpdlib/issues/329
+                instance = self.socket_map[fd]
+                self.register(fd, instance, events)
+            else:
+                raise
 
     def poll(self, timeout):
         try:
             events = self._poller.poll(timeout or -1)  # -1 waits indefinitely
-        except (IOError, select.error):  # for epoll() and poll() respectively
-            err = sys.exc_info()[1]
-            if err.args[0] == errno.EINTR:
+        except (IOError, select.error) as err:
+            # for epoll() and poll() respectively
+            if err.errno == errno.EINTR:
                 return
             raise
         # localize variable access to minimize overhead
@@ -525,12 +553,10 @@ if hasattr(select, 'poll'):
         _ERROR = select.POLLERR | select.POLLHUP | select.POLLNVAL
         _poller = select.poll
 
-        # select.poll() on py < 2.6 has no 'modify' method
-        if not hasattr(select.poll(), 'modify'):
-            def modify(self, fd, events):
-                inst = self.socket_map[fd]
-                self.unregister(fd)
-                self.register(fd, inst, events)
+        def modify(self, fd, events):
+            inst = self.socket_map[fd]
+            self.unregister(fd)
+            self.register(fd, inst, events)
 
         def poll(self, timeout):
             # poll() timeout is expressed in milliseconds
@@ -539,6 +565,44 @@ if hasattr(select, 'poll'):
             _BasePollEpoll.poll(self, timeout)
 
 
+# ===================================================================
+# --- /dev/poll - Solaris (introduced in python 3.3)
+# ===================================================================
+
+if hasattr(select, 'devpoll'):
+
+    class DevPoll(_BasePollEpoll):
+        """/dev/poll based poller (introduced in python 3.3)."""
+
+        READ = select.POLLIN
+        WRITE = select.POLLOUT
+        _ERROR = select.POLLERR | select.POLLHUP | select.POLLNVAL
+        _poller = select.devpoll
+
+        # introduced in python 3.4
+        if hasattr(select.devpoll, 'fileno'):
+            def fileno(self):
+                """Return devpoll() fd."""
+                return self._poller.fileno()
+
+        def modify(self, fd, events):
+            inst = self.socket_map[fd]
+            self.unregister(fd)
+            self.register(fd, inst, events)
+
+        def poll(self, timeout):
+            # /dev/poll timeout is expressed in milliseconds
+            if timeout is not None:
+                timeout = int(timeout * 1000)
+            _BasePollEpoll.poll(self, timeout)
+
+        # introduced in python 3.4
+        if hasattr(select.devpoll, 'close'):
+            def close(self):
+                _IOLoop.close(self)
+                self._poller.close()
+
+
 # ===================================================================
 # --- epoll() - Linux
 # ===================================================================
@@ -578,7 +642,7 @@ if hasattr(select, 'kqueue'):
 
         def fileno(self):
             """Return kqueue() fd."""
-            return self._poller.fileno()
+            return self._kqueue.fileno()
 
         def close(self):
             _IOLoop.close(self)
@@ -586,7 +650,14 @@ if hasattr(select, 'kqueue'):
 
         def register(self, fd, instance, events):
             self.socket_map[fd] = instance
-            self._control(fd, events, select.KQ_EV_ADD)
+            try:
+                self._control(fd, events, select.KQ_EV_ADD)
+            except EnvironmentError as err:
+                if err.errno == errno.EEXIST:
+                    debug("call: register(); poller raised EEXIST; ignored",
+                          self)
+                else:
+                    raise
             self._active[fd] = events
 
         def unregister(self, fd):
@@ -598,9 +669,11 @@ if hasattr(select, 'kqueue'):
             else:
                 try:
                     self._control(fd, events, select.KQ_EV_DELETE)
-                except OSError:
-                    err = sys.exc_info()[1]
-                    if err.errno != errno.EBADF:
+                except EnvironmentError as err:
+                    if err.errno in (errno.ENOENT, errno.EBADF):
+                        debug("call: unregister(); poller returned %r; "
+                              "ignoring it" % err, self)
+                    else:
                         raise
 
         def modify(self, fd, events):
@@ -612,11 +685,11 @@ if hasattr(select, 'kqueue'):
             kevents = []
             if events & self.WRITE:
                 kevents.append(select.kevent(
-                        fd, filter=select.KQ_FILTER_WRITE, flags=flags))
+                    fd, filter=select.KQ_FILTER_WRITE, flags=flags))
             if events & self.READ or not kevents:
                 # always read when there is not a write
                 kevents.append(select.kevent(
-                        fd, filter=select.KQ_FILTER_READ, flags=flags))
+                    fd, filter=select.KQ_FILTER_READ, flags=flags))
             # even though control() takes a list, it seems to return
             # EINVAL on Mac OS X (10.6) when there is more than one
             # event in the list
@@ -624,18 +697,18 @@ if hasattr(select, 'kqueue'):
                 self._kqueue.control([kevent], 0)
 
         # localize variable access to minimize overhead
-        def poll(self, timeout,
-                       _len=len,
-                       _READ=select.KQ_FILTER_READ,
-                       _WRITE=select.KQ_FILTER_WRITE,
-                       _EOF=select.KQ_EV_EOF,
-                       _ERROR=select.KQ_EV_ERROR):
+        def poll(self,
+                 timeout,
+                 _len=len,
+                 _READ=select.KQ_FILTER_READ,
+                 _WRITE=select.KQ_FILTER_WRITE,
+                 _EOF=select.KQ_EV_EOF,
+                 _ERROR=select.KQ_EV_ERROR):
             try:
                 kevents = self._kqueue.control(None, _len(self.socket_map),
                                                timeout)
-            except OSError:
-                err = sys.exc_info()[1]
-                if err.args[0] == errno.EINTR:
+            except OSError as err:
+                if err.errno == errno.EINTR:
                     return
                 raise
             for kevent in kevents:
@@ -666,13 +739,15 @@ if hasattr(select, 'kqueue'):
 # --- choose the better poller for this platform
 # ===================================================================
 
-if hasattr(select, 'epoll'):     # epoll() - Linux only
+if hasattr(select, 'epoll'):      # epoll() - Linux
     IOLoop = Epoll
-elif hasattr(select, 'kqueue'):  # kqueue() - BSD / OSX
+elif hasattr(select, 'kqueue'):   # kqueue() - BSD / OSX
     IOLoop = Kqueue
-elif hasattr(select, 'poll'):    # poll() - POSIX
+elif hasattr(select, 'devpoll'):  # /dev/poll - Solaris
+    IOLoop = DevPoll
+elif hasattr(select, 'poll'):     # poll() - POSIX
     IOLoop = Poll
-else:                            # select() - POSIX and Windows
+else:                             # select() - POSIX and Windows
     IOLoop = Select
 
 
@@ -683,18 +758,228 @@ else:                            # select() - POSIX and Windows
 # these are overridden in order to register() and unregister()
 # file descriptors against the new pollers
 
-_DISCONNECTED = frozenset((errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN,
-                           errno.ECONNABORTED, errno.EPIPE, errno.EBADF))
 
-class Acceptor(asyncore.dispatcher):
-    """Same as base asyncore.dispatcher and supposed to be used to
+class AsyncChat(asynchat.async_chat):
+    """Same as asynchat.async_chat, only working with the new IO poller
+    and being more clever in avoid registering for read events when
+    it shouldn't.
+    """
+
+    def __init__(self, sock=None, ioloop=None):
+        self.ioloop = ioloop or IOLoop.instance()
+        self._wanted_io_events = self.ioloop.READ
+        self._current_io_events = self.ioloop.READ
+        self._closed = False
+        self._closing = False
+        self._fileno = sock.fileno() if sock else None
+        self._tasks = []
+        asynchat.async_chat.__init__(self, sock)
+
+    # --- IO loop related methods
+
+    def add_channel(self, map=None, events=None):
+        assert self._fileno, repr(self._fileno)
+        events = events if events is not None else self.ioloop.READ
+        self.ioloop.register(self._fileno, self, events)
+        self._wanted_io_events = events
+        self._current_io_events = events
+
+    def del_channel(self, map=None):
+        if self._fileno is not None:
+            self.ioloop.unregister(self._fileno)
+
+    def modify_ioloop_events(self, events, logdebug=False):
+        if not self._closed:
+            assert self._fileno, repr(self._fileno)
+            if self._fileno not in self.ioloop.socket_map:
+                debug(
+                    "call: modify_ioloop_events(), fd was no longer in "
+                    "socket_map, had to register() it again", inst=self)
+                self.add_channel(events=events)
+            else:
+                if events != self._current_io_events:
+                    if logdebug:
+                        if events == self.ioloop.READ:
+                            ev = "R"
+                        elif events == self.ioloop.WRITE:
+                            ev = "W"
+                        elif events == self.ioloop.READ | self.ioloop.WRITE:
+                            ev = "RW"
+                        else:
+                            ev = events
+                        debug("call: IOLoop.modify(); setting %r IO events" % (
+                            ev), self)
+                    self.ioloop.modify(self._fileno, events)
+            self._current_io_events = events
+        else:
+            debug(
+                "call: modify_ioloop_events(), handler had already been "
+                "close()d, skipping modify()", inst=self)
+
+    # --- utils
+
+    def call_later(self, seconds, target, *args, **kwargs):
+        """Same as self.ioloop.call_later but also cancel()s the
+        scheduled function on close().
+        """
+        if '_errback' not in kwargs and hasattr(self, 'handle_error'):
+            kwargs['_errback'] = self.handle_error
+        callback = self.ioloop.call_later(seconds, target, *args, **kwargs)
+        self._tasks.append(callback)
+        return callback
+
+    # --- overridden asynchat methods
+
+    def connect(self, addr):
+        self.modify_ioloop_events(self.ioloop.WRITE)
+        asynchat.async_chat.connect(self, addr)
+
+    def connect_af_unspecified(self, addr, source_address=None):
+        """Same as connect() but guesses address family from addr.
+        Return the address family just determined.
+        """
+        assert self.socket is None
+        host, port = addr
+        err = "getaddrinfo() returned an empty list"
+        info = socket.getaddrinfo(host, port, socket.AF_UNSPEC,
+                                  socket.SOCK_STREAM, 0, socket.AI_PASSIVE)
+        for res in info:
+            self.socket = None
+            af, socktype, proto, canonname, sa = res
+            try:
+                self.create_socket(af, socktype)
+                if source_address:
+                    if source_address[0].startswith('::ffff:'):
+                        # In this scenario, the server has an IPv6 socket, but
+                        # the remote client is using IPv4 and its address is
+                        # represented as an IPv4-mapped IPv6 address which
+                        # looks like this ::ffff:151.12.5.65, see:
+                        # http://en.wikipedia.org/wiki/IPv6\
+                        #     IPv4-mapped_addresses
+                        # http://tools.ietf.org/html/rfc3493.html#section-3.7
+                        # We truncate the first bytes to make it look like a
+                        # common IPv4 address.
+                        source_address = (source_address[0][7:],
+                                          source_address[1])
+                    self.bind(source_address)
+                self.connect((host, port))
+            except socket.error as err:
+                if self.socket is not None:
+                    self.socket.close()
+                    self.del_channel()
+                    self.socket = None
+                continue
+            break
+        if self.socket is None:
+            self.del_channel()
+            raise socket.error(err)
+        return af
+
+    # send() and recv() overridden as a fix around various bugs:
+    # - http://bugs.python.org/issue1736101
+    # - https://github.com/giampaolo/pyftpdlib/issues/104
+    # - https://github.com/giampaolo/pyftpdlib/issues/109
+
+    def send(self, data):
+        try:
+            return self.socket.send(data)
+        except socket.error as err:
+            debug("call: send(), err: %s" % err, inst=self)
+            if err.errno in _ERRNOS_RETRY:
+                return 0
+            elif err.errno in _ERRNOS_DISCONNECTED:
+                self.handle_close()
+                return 0
+            else:
+                raise
+
+    def recv(self, buffer_size):
+        try:
+            data = self.socket.recv(buffer_size)
+        except socket.error as err:
+            debug("call: recv(), err: %s" % err, inst=self)
+            if err.errno in _ERRNOS_DISCONNECTED:
+                self.handle_close()
+                return b''
+            elif err.errno in _ERRNOS_RETRY:
+                raise RetryError
+            else:
+                raise
+        else:
+            if not data:
+                # a closed connection is indicated by signaling
+                # a read condition, and having recv() return 0.
+                self.handle_close()
+                return b''
+            else:
+                return data
+
+    def handle_read(self):
+        try:
+            asynchat.async_chat.handle_read(self)
+        except RetryError:
+            # This can be raised by (the overridden) recv().
+            pass
+
+    def initiate_send(self):
+        asynchat.async_chat.initiate_send(self)
+        if not self._closed:
+            # if there's still data to send we want to be ready
+            # for writing, else we're only intereseted in reading
+            if not self.producer_fifo:
+                wanted = self.ioloop.READ
+            else:
+                # In FTPHandler, we also want to listen for user input
+                # hence the READ. DTPHandler has its own initiate_send()
+                # which will either READ or WRITE.
+                wanted = self.ioloop.READ | self.ioloop.WRITE
+            if self._wanted_io_events != wanted:
+                self.ioloop.modify(self._fileno, wanted)
+                self._wanted_io_events = wanted
+        else:
+            debug("call: initiate_send(); called with no connection",
+                  inst=self)
+
+    def close_when_done(self):
+        if len(self.producer_fifo) == 0:
+            self.handle_close()
+        else:
+            self._closing = True
+            asynchat.async_chat.close_when_done(self)
+
+    def close(self):
+        if not self._closed:
+            self._closed = True
+            try:
+                asynchat.async_chat.close(self)
+            finally:
+                for fun in self._tasks:
+                    try:
+                        fun.cancel()
+                    except Exception:
+                        logger.error(traceback.format_exc())
+                self._tasks = []
+                self._closed = True
+                self._closing = False
+                self.connected = False
+
+
+class Connector(AsyncChat):
+    """Same as base AsyncChat and supposed to be used for
+    clients.
+    """
+
+    def add_channel(self, map=None, events=None):
+        AsyncChat.add_channel(self, map=map, events=self.ioloop.WRITE)
+
+
+class Acceptor(AsyncChat):
+    """Same as base AsyncChat and supposed to be used to
     accept new connections.
     """
 
-    def __init__(self, ioloop=None):
-        self.ioloop = ioloop or IOLoop.instance()
-        self._fileno = None  # py < 2.6
-        asyncore.dispatcher.__init__(self)
+    def add_channel(self, map=None, events=None):
+        AsyncChat.add_channel(self, map=map, events=self.ioloop.READ)
 
     def bind_af_unspecified(self, addr):
         """Same as bind() but guesses address family from addr.
@@ -719,8 +1004,8 @@ class Acceptor(asyncore.dispatcher):
                 self.create_socket(af, socktype)
                 self.set_reuse_addr()
                 self.bind(sa)
-            except socket.error:
-                err = sys.exc_info()[1]
+            except socket.error as _:
+                err = _
                 if self.socket is not None:
                     self.socket.close()
                     self.del_channel()
@@ -732,14 +1017,8 @@ class Acceptor(asyncore.dispatcher):
             raise socket.error(err)
         return af
 
-    def add_channel(self, map=None):
-        self.ioloop.register(self._fileno, self, self.ioloop.READ)
-
-    def del_channel(self, map=None):
-        self.ioloop.unregister(self._fileno)
-
     def listen(self, num):
-        asyncore.dispatcher.listen(self, num)
+        AsyncChat.listen(self, num)
         # XXX - this seems to be necessary, otherwise kqueue.control()
         # won't return listening fd events
         try:
@@ -753,12 +1032,15 @@ class Acceptor(asyncore.dispatcher):
             sock, addr = self.accept()
         except TypeError:
             # sometimes accept() might return None (see issue 91)
+            debug("call: handle_accept(); accept() returned None", self)
             return
-        except socket.error:
-            err = sys.exc_info()[1]
+        except socket.error as err:
             # ECONNABORTED might be thrown on *BSD (see issue 105)
-            if err.args[0] != errno.ECONNABORTED:
+            if err.errno != errno.ECONNABORTED:
                 raise
+            else:
+                debug("call: handle_accept(); accept() returned ECONNABORTED",
+                      self)
         else:
             # sometimes addr == None instead of (ip, port) (see issue 104)
             if addr is not None:
@@ -772,129 +1054,3 @@ class Acceptor(asyncore.dispatcher):
     if (os.name in ('nt', 'ce')) or (sys.platform == 'cygwin'):
         def set_reuse_addr(self):
             pass
-
-
-class Connector(Acceptor):
-    """Same as base asyncore.dispatcher and supposed to be used for
-    clients.
-    """
-
-    def connect_af_unspecified(self, addr, source_address=None):
-        """Same as connect() but guesses address family from addr.
-        Return the address family just determined.
-        """
-        assert self.socket is None
-        host, port = addr
-        err = "getaddrinfo() returned an empty list"
-        info = socket.getaddrinfo(host, port, socket.AF_UNSPEC,
-                                  socket.SOCK_STREAM, 0, socket.AI_PASSIVE)
-        for res in info:
-            self.socket = None
-            af, socktype, proto, canonname, sa = res
-            try:
-                self.create_socket(af, socktype)
-                if source_address:
-                    if source_address[0].startswith('::ffff:'):
-                        # In this scenario, the server has an IPv6 socket, but
-                        # the remote client is using IPv4 and its address is
-                        # represented as an IPv4-mapped IPv6 address which
-                        # looks like this ::ffff:151.12.5.65, see:
-                        # http://en.wikipedia.org/wiki/IPv6#IPv4-mapped_addresses
-                        # http://tools.ietf.org/html/rfc3493.html#section-3.7
-                        # We truncate the first bytes to make it look like a
-                        # common IPv4 address.
-                        source_address = (source_address[0][7:],
-                                          source_address[1])
-                    self.bind(source_address)
-                self.connect((host, port))
-            except socket.error:
-                err = sys.exc_info()[1]
-                if self.socket is not None:
-                    self.socket.close()
-                    self.del_channel()
-                    self.socket = None
-                continue
-            break
-        if self.socket is None:
-            self.del_channel()
-            raise socket.error(err)
-        return af
-
-    def add_channel(self, map=None):
-        self.ioloop.register(self._fileno, self, self.ioloop.WRITE)
-
-
-class AsyncChat(asynchat.async_chat):
-    """Same as asynchat.async_chat, only working with the new IO poller
-    and being more clever in avoid registering for read events when
-    it shouldn't.
-    """
-
-    def __init__(self, sock, ioloop=None):
-        self.ioloop = ioloop or IOLoop.instance()
-        self._current_io_events = self.ioloop.READ
-        self._closed = False
-        self._closing = False
-        asynchat.async_chat.__init__(self, sock)
-
-    def add_channel(self, map=None, events=None):
-        self.ioloop.register(self._fileno, self, events or self.ioloop.READ)
-
-    def del_channel(self, map=None):
-        self.ioloop.unregister(self._fileno)
-
-    # send() and recv() overridden as a fix around various bugs:
-    # - http://bugs.python.org/issue1736101
-    # - http://code.google.com/p/pyftpdlib/issues/detail?id=104
-    # - http://code.google.com/p/pyftpdlib/issues/detail?id=109
-
-    def send(self, data):
-        try:
-            return self.socket.send(data)
-        except socket.error:
-            why = sys.exc_info()[1]
-            if why.args[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
-                return 0
-            elif why.args[0] in _DISCONNECTED:
-                self.handle_close()
-                return 0
-            else:
-                raise
-
-    def recv(self, buffer_size):
-        try:
-            data = self.socket.recv(buffer_size)
-            if not data:
-                # a closed connection is indicated by signaling
-                # a read condition, and having recv() return 0.
-                self.handle_close()
-                return b('')
-            else:
-                return data
-        except socket.error:
-            why = sys.exc_info()[1]
-            if why.args[0] in _DISCONNECTED:
-                self.handle_close()
-                return b('')
-            else:
-                raise
-
-    def initiate_send(self):
-        asynchat.async_chat.initiate_send(self)
-        if not self._closed:
-            # if there's still data to send we want to be ready
-            # for writing, else we're only intereseted in reading
-            if not self.producer_fifo:
-                wanted = self.ioloop.READ
-            else:
-                wanted = self.ioloop.READ | self.ioloop.WRITE
-            if self._current_io_events != wanted:
-                self.ioloop.modify(self._fileno, wanted)
-                self._current_io_events = wanted
-
-    def close_when_done(self):
-        if len(self.producer_fifo) == 0:
-            self.handle_close()
-        else:
-            self._closing = True
-            asynchat.async_chat.close_when_done(self)
